@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -30,7 +30,7 @@
 #ifdef VMS
 #include <unixlib.h>
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
@@ -135,6 +135,8 @@ static XtActionsRec actionsListCubes[] =
 
 static XtResource resourcesCubes[] =
 {
+	{XtNuserName, XtCUserName, XtRString, sizeof (String),
+	 XtOffset(CubesWidget, cubes.username), XtRString, "nobody"},
 	{XtNforeground, XtCForeground, XtRPixel, sizeof (Pixel),
     XtOffset(CubesWidget, cubes.foreground), XtRString, XtDefaultForeground},
 	{XtNbrickColor, XtCColor, XtRPixel, sizeof (Pixel),
