@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1995 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1995 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -30,7 +30,7 @@
 #ifdef VMS
 #include <unixlib.h>
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
@@ -578,7 +578,7 @@ WriteDino(DinoWidget w, XEvent * event, char **args, int nArgs)
 	if ((fp = fopen(DATAFILE, "w")) == NULL)
 		(void) printf("Can not write to %s.\n", DATAFILE);
 	else {
-		(void) fprintf(fp, "dino%c %d\n", SYMBOL, w->dino.mode);
+		(void) fprintf(fp, "mode%c %d\n", SYMBOL, w->dino.mode);
 		(void) fprintf(fp, "orient%c %d\n", SYMBOL, (w->dino.orient) ? 1 : 0);
 		(void) fprintf(fp, "practice%c %d\n", SYMBOL, (w->dino.practice) ? 1 : 0);
 		(void) fprintf(fp, "moves%c %d\n", SYMBOL, NumMoves());
