@@ -6,7 +6,7 @@
 #
 ###
 #
-#  Copyright (c) 1995 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1995 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -134,6 +134,8 @@ static XtActionsRec actionsListDino3D[] =
 
 static XtResource resourcesDino3D[] =
 {
+	{XtNuserName, XtCUserName, XtRString, sizeof (String),
+	 XtOffset(DinoWidget, dino.username), XtRString, "nobody"},
 	{XtNfaceColor0, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(DinoWidget, dino.faceName[0]), XtRString, "Orange"},
 	{XtNfaceColor1, XtCLabel, XtRString, sizeof (String),
