@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -30,7 +30,7 @@
 #ifdef VMS
 #include <unixlib.h>
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
@@ -163,6 +163,8 @@ static XtActionsRec actionsListHexagons[] =
 
 static XtResource resourcesHexagons[] =
 {
+	{XtNuserName, XtCUserName, XtRString, sizeof (String),
+	 XtOffset(HexagonsWidget, hexagons.username), XtRString, "nobody"},
 	{XtNforeground, XtCForeground, XtRPixel, sizeof (Pixel),
 	 XtOffset(HexagonsWidget, hexagons.foreground), XtRString,
 	 XtDefaultForeground},
@@ -209,7 +211,7 @@ HexagonsClassRec hexagonsClassRec =
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
-		(XtWidgetProc) DestroyHexagons,	/* destroy */
+		(XtWidgetProc) DestroyHexagons,		/* destroy */
 		(XtWidgetProc) ResizeHexagons,	/* resize */
 		(XtExposeProc) ExposeHexagons,	/* expose */
 		(XtSetValuesFunc) SetValuesHexagons,	/* set values */
@@ -252,7 +254,7 @@ static XPoint hexagonUnit[MAXORIENT][7] =
 };
 static XPoint hexagonList[MAXORIENT][7];
 
-static void 
+static void
 InitializeHexagons(Widget request, Widget new)
 {
 	HexagonsWidget w = (HexagonsWidget) new;
@@ -279,7 +281,7 @@ InitializeHexagons(Widget request, Widget new)
 	ResizeHexagons(w);
 }
 
-static void 
+static void
 DestroyHexagons(Widget old)
 {
 	HexagonsWidget w = (HexagonsWidget) old;
@@ -291,7 +293,7 @@ DestroyHexagons(Widget old)
 	XtRemoveCallbacks(old, XtNselectCallback, w->hexagons.select);
 }
 
-static void 
+static void
 ResizeHexagons(HexagonsWidget w)
 {
 	double      sqrt_3 = 1.73205080756887729352744634150587237;
@@ -330,7 +332,7 @@ ResizeHexagons(HexagonsWidget w)
 	ResizeTiles(w);
 }
 
-static void 
+static void
 ExposeHexagons(Widget new, XEvent * event, Region region)
 		 /* Not used */
 {
@@ -342,7 +344,7 @@ ExposeHexagons(Widget new, XEvent * event, Region region)
 	}
 }
 
-static Boolean 
+static      Boolean
 SetValuesHexagons(Widget current, Widget request, Widget new)
 {
 	HexagonsWidget c = (HexagonsWidget) current, w = (HexagonsWidget) new;
@@ -404,14 +406,14 @@ SetValuesHexagons(Widget current, Widget request, Widget new)
 	return (redraw);
 }
 
-static void 
+static void
 QuitHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	XtCloseDisplay(XtDisplay(w));
 	exit(0);
 }
 
-static void 
+static void
 SelectHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	int         pos, row;
@@ -438,7 +440,7 @@ SelectHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 		w->hexagons.currentPosition = -1;
 }
 
-static void 
+static void
 ReleaseHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	int         pos, row, space;
@@ -472,20 +474,20 @@ ReleaseHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 	w->hexagons.currentPosition = -1;
 }
 
-static void 
+static void
 RandomizeHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	RandomizeTiles(w);
 }
 
-static void 
+static void
 RandomizeHexagonsMaybe(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	if (!w->hexagons.started)
 		RandomizeTiles(w);
 }
 
-static void 
+static void
 GetHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	FILE       *fp;
@@ -530,7 +532,7 @@ GetHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 	}
 }
 
-static void 
+static void
 WriteHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	FILE       *fp;
@@ -548,7 +550,7 @@ WriteHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 	}
 }
 
-static void 
+static void
 UndoHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	if (MadeMoves()) {
@@ -565,13 +567,13 @@ UndoHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 	}
 }
 
-static void 
+static void
 SolveHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	/* SolveTiles(w); *//* Sorry, unimplemented */
 }
 
-static void 
+static void
 IncrementHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	hexagonsCallbackStruct cb;
@@ -580,7 +582,7 @@ IncrementHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 	XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
 }
 
-static void 
+static void
 DecrementHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	hexagonsCallbackStruct cb;
@@ -591,7 +593,7 @@ DecrementHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 	XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
 }
 
-static void 
+static void
 CornerHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	hexagonsCallbackStruct cb;
@@ -600,43 +602,43 @@ CornerHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 	XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
 }
 
-static void 
+static void
 MoveHexagonsTl(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	(void) MoveHexagons(w, TL);
 }
 
-static void 
+static void
 MoveHexagonsTr(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	(void) MoveHexagons(w, TR);
 }
 
-static void 
+static void
 MoveHexagonsLeft(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	(void) MoveHexagons(w, LEFT);
 }
 
-static void 
+static void
 MoveHexagonsRight(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	(void) MoveHexagons(w, RIGHT);
 }
 
-static void 
+static void
 MoveHexagonsBl(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	(void) MoveHexagons(w, BL);
 }
 
-static void 
+static void
 MoveHexagonsBr(HexagonsWidget w, XEvent * event, char **args, int nArgs)
 {
 	(void) MoveHexagons(w, BR);
 }
 
-static int 
+static int
 MoveHexagons(HexagonsWidget w, int direction)
 {
 	hexagonsCallbackStruct cb;
@@ -657,7 +659,7 @@ MoveHexagons(HexagonsWidget w, int direction)
 	return TRUE;
 }
 
-int 
+int
 MoveHexagonsDir(HexagonsWidget w, int direction)
 {
 	hexagonsCallbackStruct cb;
@@ -671,7 +673,7 @@ MoveHexagonsDir(HexagonsWidget w, int direction)
 	return FALSE;
 }
 
-static int 
+static int
 PositionToTile(HexagonsWidget w, int x, int y, int *row)
 {
 	int         i, j, k, modI, modJ;
@@ -712,7 +714,7 @@ PositionToTile(HexagonsWidget w, int x, int y, int *row)
 	return i;
 }
 
-static void 
+static void
 SelectCornerTiles(HexagonsWidget w, int space)
 {
 	int         orient;
@@ -750,7 +752,7 @@ SelectCornerTiles(HexagonsWidget w, int space)
 	}
 }
 
-static void 
+static void
 SelectNoCornTiles(HexagonsWidget w)
 {
 	hexagonsCallbackStruct cb;
@@ -823,7 +825,7 @@ SelectNoCornTiles(HexagonsWidget w)
 	}
 }
 
-static void 
+static void
 CheckTiles(HexagonsWidget w)
 {
 	char        buf[121];
@@ -838,7 +840,7 @@ CheckTiles(HexagonsWidget w)
 	w->hexagons.base = 10;
 }
 
-static void 
+static void
 ResetTiles(HexagonsWidget w)
 {
 	int         i;
@@ -879,7 +881,7 @@ ResetTiles(HexagonsWidget w)
 	w->hexagons.started = FALSE;
 }
 
-static void 
+static void
 ResizeTiles(HexagonsWidget w)
 {
 	int         i;
@@ -898,7 +900,7 @@ ResizeTiles(HexagonsWidget w)
 	w->hexagons.digitOffset.y = 4;
 }
 
-static int 
+static int
 MovableCornerTiles(HexagonsWidget w, int direction, int *position, int *posRow, int *space)
 {
 	int         spaceType, highest, side = -1;
@@ -960,7 +962,7 @@ MovableCornerTiles(HexagonsWidget w, int direction, int *position, int *posRow,
 	return TRUE;
 }
 
-static void 
+static void
 MoveNoTiles(HexagonsWidget w)
 {
 	hexagonsCallbackStruct cb;
@@ -969,7 +971,7 @@ MoveNoTiles(HexagonsWidget w)
 	XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
 }
 
-static int 
+static int
 MoveTilesDir(HexagonsWidget w, int direction)
 {
 	if (w->hexagons.corners)
@@ -979,7 +981,7 @@ MoveTilesDir(HexagonsWidget w, int direction)
 }
 
 
-static int 
+static int
 MoveCornerTilesDir(HexagonsWidget w, int direction)
 {
 	int         position, posRow, space;
@@ -992,7 +994,7 @@ MoveCornerTilesDir(HexagonsWidget w, int direction)
 	return FALSE;
 }
 
-static int 
+static int
 MoveNoCornTilesDir(HexagonsWidget w, int direction)
 {
 	switch (direction) {
@@ -1050,7 +1052,7 @@ MoveNoCornTilesDir(HexagonsWidget w, int direction)
 	return FALSE;
 }
 
-static void 
+static void
 RandomizeTiles(HexagonsWidget w)
 {
 	hexagonsCallbackStruct cb;
@@ -1115,7 +1117,7 @@ RandomizeTiles(HexagonsWidget w)
 	}
 }
 
-static void 
+static void
 MoveCornerTiles(HexagonsWidget w, int from, int posRow, int space)
 {
 	int         tempTile;
@@ -1133,7 +1135,7 @@ MoveCornerTiles(HexagonsWidget w, int from, int posRow, int space)
 		 w->hexagons.spacePosition[space], FALSE);
 }
 
-static void 
+static void
 MoveNoCornTiles(HexagonsWidget w, int from, int posRow)
 {
 	int         tempTile;
@@ -1152,7 +1154,7 @@ MoveNoCornTiles(HexagonsWidget w, int from, int posRow)
 		 w->hexagons.spacePosition[HIGH], FALSE);
 }
 
-static int 
+static int
 ExchangeTiles(HexagonsWidget w, int pos1, int pos2)
 {
 	int         tempTile;
@@ -1167,7 +1169,7 @@ ExchangeTiles(HexagonsWidget w, int pos1, int pos2)
 }
 
 #ifdef DEBUG
-static int 
+static int
 WithinFrame(HexagonsWidget w, int x, int y, int dx, int dy)
 {
 	return
@@ -1182,7 +1184,7 @@ WithinFrame(HexagonsWidget w, int x, int y, int dx, int dy)
 }
 #endif
 
-static int 
+static int
 NextToWall(HexagonsWidget w, int pos, int posRow, int spaceType)
 {
 	switch (spaceType) {
@@ -1220,7 +1222,7 @@ NextToWall(HexagonsWidget w, int pos, int posRow, int spaceType)
 	return (-2);		/*Unknown space formation. */
 }
 
-static int 
+static int
 TileNextToSpace(HexagonsWidget w, int rowType, int direction)
 {
 	if (direction == HIGH) {
@@ -1256,7 +1258,7 @@ TileNextToSpace(HexagonsWidget w, int rowType, int direction)
 	}
 }
 
-static int 
+static int
 FindTileTriangle(HexagonsWidget w, int pI, int pJ, int pK, int rI, int rJ, int rK)
 {
 	int         found = TRUE, temp = 0, k = 0, row1 = 0, row2 = 0, pos;
@@ -1311,7 +1313,7 @@ FindTileTriangle(HexagonsWidget w, int pI, int pJ, int pK, int rI, int rJ, int r
 	return (0);
 }
 
-static int 
+static int
 FindDir(HexagonsWidget w, int posTile, int posSpace, int rowTile, int rowSpace)
 {
 	if (rowTile == rowSpace) {
@@ -1333,7 +1335,7 @@ FindDir(HexagonsWidget w, int posTile, int posSpace, int rowTile, int rowSpace)
 	}
 }
 
-static int 
+static int
 FindSpaceType(HexagonsWidget w, int pos1, int pos2, int row1, int row2)
 {
 	if (row1 == row2 && (pos1 == pos2 + 1 || pos1 == pos2 - 1))
@@ -1358,7 +1360,7 @@ FindSpaceType(HexagonsWidget w, int pos1, int pos2, int row1, int row2)
 	return (-1);
 }
 
-static void 
+static void
 FindMovableTile(HexagonsWidget w, int pos, int posRow, int spaceType, int side, int *tilePos, int *tileRow)
 {
 	switch (spaceType) {
@@ -1402,7 +1404,7 @@ FindMovableTile(HexagonsWidget w, int pos, int posRow, int spaceType, int side,
 	}
 }
 
-static void 
+static void
 DrawFrame(HexagonsWidget w, GC gc)
 {
 	int         sumX, sumY, sumX4, sum3X4, sumY2, offsetX, offsetY;
@@ -1433,7 +1435,7 @@ DrawFrame(HexagonsWidget w, GC gc)
 		  offsetX, sumY2, sumX4, offsetY);
 }
 
-void 
+void
 DrawAllTiles(HexagonsWidget w, GC tileGC, GC borderGC)
 {
 	int         k;
@@ -1445,7 +1447,7 @@ DrawAllTiles(HexagonsWidget w, GC tileGC, GC borderGC)
 			DrawTile(w, tileGC, borderGC, k, FALSE);
 }
 
-static void 
+static void
 DrawTile(HexagonsWidget w, GC tileGC, GC borderGC, int pos, int offset)
 {
 	int         dx, dy, k = Row(w, pos), orient = (w->hexagons.corners) ? 1 : 0;
@@ -1485,7 +1487,7 @@ DrawTile(HexagonsWidget w, GC tileGC, GC borderGC, int pos, int offset)
 }
 
 #ifdef DEBUG
-static int 
+static int
 PositionInRow(w, position, posRow)
 	HexagonsWidget w;
 	int         position, posRow;
@@ -1499,17 +1501,17 @@ PositionInRow(w, position, posRow)
 }
 #endif
 
-static int 
+static int
 PositionFromRow(HexagonsWidget w, int rowPosition, int posRow)
 {
 	return (posRow <= w->hexagons.size - 1) ?
 		(w->hexagons.size * posRow + posRow * (posRow - 1) / 2 + rowPosition) :
 		(w->hexagons.size * (posRow - w->hexagons.size) + 4 *
 		 w->hexagons.size * (w->hexagons.size - 1) / 2 - (2 * w->hexagons.size -
-	  posRow - 2) * (2 * w->hexagons.size - posRow - 1) / 2 + 1 + rowPosition);
+								  posRow - 2) * (2 * w->hexagons.size - posRow - 1) / 2 + 1 + rowPosition);
 }
 
-int 
+int
 Row(HexagonsWidget w, int pos)
 {
 	return (pos <= w->hexagons.sizeCenter) ?
@@ -1520,7 +1522,7 @@ Row(HexagonsWidget w, int pos)
 }
 
 /* Passing row so there is no sqrt calculation again */
-int 
+int
 TrBl(HexagonsWidget w, int pos, int posRow)
 {
 	return (pos <= w->hexagons.sizeCenter) ?
@@ -1528,11 +1530,11 @@ TrBl(HexagonsWidget w, int pos, int posRow)
 		(posRow + w->hexagons.size) * (posRow + w->hexagons.size - 1) / 2 :
 		2 * w->hexagons.size - 2 - (w->hexagons.sizeSize - 1 +
 			w->hexagons.size * (w->hexagons.size - 1) / 2 - pos -
-	  (3 * w->hexagons.size - posRow - 2) * (3 * w->hexagons.size - posRow - 3) /
+					    (3 * w->hexagons.size - posRow - 2) * (3 * w->hexagons.size - posRow - 3) /
 					    2);
 }
 
-int 
+int
 TlBr(HexagonsWidget w, int pos, int posRow)
 {
 	return (pos <= w->hexagons.sizeCenter) ?
@@ -1540,12 +1542,12 @@ TlBr(HexagonsWidget w, int pos, int posRow)
 	 (posRow + w->hexagons.size + 1) * (posRow + w->hexagons.size) / 2) :
 		2 * w->hexagons.size - 1 + (w->hexagons.sizeSize - 1 +
 			w->hexagons.size * (w->hexagons.size - 1) / 2 - pos -
-    (3 * w->hexagons.size - posRow - 1) * (3 * w->hexagons.size - posRow - 2) /
+					    (3 * w->hexagons.size - posRow - 1) * (3 * w->hexagons.size - posRow - 2) /
 					    2);
 }
 
 /* This is fast for small i, a -1 is returned for negative i. */
-static int 
+static int
 Sqrt(int i)
 {
 	int         j = 0;
@@ -1555,7 +1557,7 @@ Sqrt(int i)
 	return (j - 1);
 }
 
-Boolean 
+Boolean
 CheckSolved(HexagonsWidget w)
 {
 	int         i;
@@ -1566,7 +1568,7 @@ CheckSolved(HexagonsWidget w)
 	return TRUE;
 }
 
-static void 
+static void
 swap(int *a, int *b)
 {
 	int         c;
