@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -30,7 +30,7 @@
 #ifdef VMS
 #include <unixlib.h>
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
@@ -136,6 +136,8 @@ static XtActionsRec actionsListMlink[] =
 
 static XtResource resourcesMlink[] =
 {
+	{XtNuserName, XtCUserName, XtRString, sizeof (String),
+	 XtOffset(MlinkWidget, mlink.username), XtRString, "nobody"},
 	{XtNfaceColor0, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(MlinkWidget, mlink.faceName[0]), XtRString, "White"},
 	{XtNfaceColor1, XtCLabel, XtRString, sizeof (String),
@@ -155,7 +157,7 @@ static XtResource resourcesMlink[] =
 	{XtNforeground, XtCBackground, XtRPixel, sizeof (Pixel),
     XtOffset(MlinkWidget, mlink.foreground), XtRString, XtDefaultBackground},
 	{XtNtileColor, XtCForeground, XtRPixel, sizeof (Pixel),
-    XtOffset(MlinkWidget, mlink.tileColor), XtRString, XtDefaultForeground},
+     XtOffset(MlinkWidget, mlink.tileColor), XtRString, XtDefaultForeground},
 	{XtNtileBorder, XtCBackground, XtRPixel, sizeof (Pixel),
    XtOffset(MlinkWidget, mlink.borderColor), XtRString, XtDefaultBackground},
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
@@ -1102,7 +1104,7 @@ static void
 DrawTile(MlinkWidget w, GC tileGC, GC borderGC, int pos, int offset)
 {
 	int         dx, dy, tile = w->mlink.tileOfPosition[pos];
-	GC gc = w->mlink.faceGC[((tile - 1) / w->mlink.tiles + 1) % w->mlink.faces];
+	GC          gc = w->mlink.faceGC[((tile - 1) / w->mlink.tiles + 1) % w->mlink.faces];
 
 	dx = Row(w, pos) * w->mlink.offset.x + w->mlink.delta.x +
 		w->mlink.puzzleOffset.x + offset;
