@@ -6,7 +6,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -31,14 +31,20 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 #ifdef VMS
 #include <unixlib.h>
-#define getlogin cuserid
+#define getlogin() cuserid(NULL)
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
+#if HAVE_FCNTL_H
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#endif
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
@@ -65,6 +71,15 @@
 
 #define MAXTILES 8
 #define MAXRECORD 32767
+#define FILENAMELEN 1024
+#define USERNAMELEN 128
+#define NOACCESS "noaccess"
+#define NOBODY "nobody"
+
+typedef struct {
+	int         score;
+	char        name[USERNAMELEN];
+} GameRecord;
 
 static void Initialize(Widget w);
 static void CallbackMlink(Widget w, caddr_t clientData,
@@ -73,6 +88,7 @@ static void CallbackMlink(Widget w, caddr_t clientData,
 static void PrintRecord(int tiles, int faces, Boolean orient, Boolean middle);
 static int  HandleSolved(int counter, int tiles, int faces, Boolean orient,
 			 Boolean middle);
+static void InitRecords(void);
 static void ReadRecords(void);
 static void WriteRecords(void);
 
@@ -89,9 +105,11 @@ static void MiddleToggle(Widget w, XtPointer clientData,
 static Arg  arg[2];
 static Widget moves, record, message, mlink, orientSwitch, middleSwitch,
             tile, face;
-static int  mlinkRecord[2][2][MAXFACES - MINFACES + 1][MAXTILES - MINTILES + 1],
-            movesDsp = 0;
+static GameRecord mlinkRecord[2][2][MAXFACES - MINFACES + 1]
+[MAXTILES - MINTILES + 1];
+static int  movesDsp = 0;
 static char messageDsp[128] = "Welcome";
+static char usernameDsp[USERNAMELEN] = "";
 static char buff[256];
 
 static void
@@ -101,32 +119,30 @@ Usage(void)
 	(void) fprintf(stderr,
 	     "\t[-geometry [{width}][x{height}][{+-}{xoff}[{+-}{yoff}]]]\n");
 	(void) fprintf(stderr,
-		"\t[-display [{host}]:[{vs}]][-fg {color}] [-bg {color}]\n");
+		       "\t[-display [{host}]:[{vs}]] [-mono]\n");
 	(void) fprintf(stderr,
-		       "\t[-mono] [-tile {color}] [-{border|bd} {color}]\n");
+		"\t[-{foreground|fg} {color}] [-{background|bg} {color}]\n");
+	(void) fprintf(stderr,
+		       "\t[-{border|bd} {color}] [-tile {color}]\n");
+	(void) fprintf(stderr,
+		       "\t[-face{0|1|2|3|4|5|6|7} {color}]\n");
 	(void) fprintf(stderr,
 	    "\t[-tiles {int}] [-faces {int}] [-[no]orient] [-[no]middle]\n");
 	(void) fprintf(stderr,
-		       "\t[-face{0|1|2|3|4|5|6|7} {color}]\n");
+		       "\t[-username {string}]\n");
 	exit(1);
 }
 
 static XrmOptionDescRec options[] =
 {
-	{"-fg", "*mlink.Foreground", XrmoptionSepArg, NULL},
-	{"-bg", "*Background", XrmoptionSepArg, NULL},
-	{"-foreground", "*mlink.Foreground", XrmoptionSepArg, NULL},
-	{"-background", "*Background", XrmoptionSepArg, NULL},
-	{"-tile", "*mlink.tileColor", XrmoptionSepArg, NULL},
-	{"-border", "*mlink.tileBorder", XrmoptionSepArg, NULL},
-	{"-bd", "*mlink.tileBorder", XrmoptionSepArg, NULL},
-	{"-tiles", "*mlink.tiles", XrmoptionSepArg, NULL},
-	{"-faces", "*mlink.faces", XrmoptionSepArg, NULL},
-	{"-orient", "*mlink.orient", XrmoptionNoArg, "TRUE"},
-	{"-noorient", "*mlink.orient", XrmoptionNoArg, "FALSE"},
-	{"-middle", "*mlink.middle", XrmoptionNoArg, "TRUE"},
-	{"-nomiddle", "*mlink.middle", XrmoptionNoArg, "FALSE"},
 	{"-mono", "*mlink.mono", XrmoptionNoArg, "TRUE"},
+	{"-fg", "*mlink.Foreground", XrmoptionSepArg, NULL},
+	{"-foreground", "*mlink.Foreground", XrmoptionSepArg, NULL},
+	{"-bg", "*Background", XrmoptionSepArg, NULL},
+	{"-background", "*Background", XrmoptionSepArg, NULL},
+	{"-bd", "*mlink.tileBorder", XrmoptionSepArg, NULL},
+	{"-border", "*mlink.tileBorder", XrmoptionSepArg, NULL},
+	{"-tile", "*mlink.tileColor", XrmoptionSepArg, NULL},
 	{"-face0", "*mlink.faceColor0", XrmoptionSepArg, NULL},
 	{"-face1", "*mlink.faceColor1", XrmoptionSepArg, NULL},
 	{"-face2", "*mlink.faceColor2", XrmoptionSepArg, NULL},
@@ -134,7 +150,14 @@ static XrmOptionDescRec options[] =
 	{"-face4", "*mlink.faceColor4", XrmoptionSepArg, NULL},
 	{"-face5", "*mlink.faceColor5", XrmoptionSepArg, NULL},
 	{"-face6", "*mlink.faceColor6", XrmoptionSepArg, NULL},
-	{"-face7", "*mlink.faceColor7", XrmoptionSepArg, NULL}
+	{"-face7", "*mlink.faceColor7", XrmoptionSepArg, NULL},
+	{"-tiles", "*mlink.tiles", XrmoptionSepArg, NULL},
+	{"-faces", "*mlink.faces", XrmoptionSepArg, NULL},
+	{"-orient", "*mlink.orient", XrmoptionNoArg, "TRUE"},
+	{"-noorient", "*mlink.orient", XrmoptionNoArg, "FALSE"},
+	{"-middle", "*mlink.middle", XrmoptionNoArg, "TRUE"},
+	{"-nomiddle", "*mlink.middle", XrmoptionNoArg, "FALSE"},
+	{"-username", "*mlink.userName", XrmoptionSepArg, NULL}
 };
 
 int
@@ -263,11 +286,13 @@ Initialize(Widget w)
 {
 	int         tiles, faces;
 	Boolean     orient, middle;
+	String      username;
 
 	XtVaSetValues(w,
 		      XtNstart, FALSE,
 		      NULL);
 	XtVaGetValues(w,
+		      XtNuserName, &username,
 		      XtNtiles, &tiles,
 		      XtNfaces, &faces,
 		      XtNorient, &orient,
@@ -278,7 +303,16 @@ Initialize(Widget w)
 	XmScaleSetValue(face, faces);
 	XmToggleButtonSetState(orientSwitch, orient, True);
 	XmToggleButtonSetState(middleSwitch, middle, True);
+	InitRecords();
 	ReadRecords();
+	(void) strcpy(usernameDsp, username);
+	if (!strcmp(usernameDsp, "") || !strcmp(usernameDsp, NOACCESS) ||
+	    !strcmp(usernameDsp, NOBODY)) {
+		/* The NOACCESS is not necasary, but it stops people from being cute. */
+		(void) sprintf(usernameDsp, "%s", getlogin());
+		if (!strcmp(usernameDsp, "") || !strcmp(usernameDsp, NOACCESS))
+			(void) sprintf(usernameDsp, "%s", NOBODY);	/* It really IS nobody */
+	}
 	PrintRecord(tiles, faces, orient, middle);
 }
 
@@ -318,7 +352,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 			return;
 		case MLINK_SOLVED:
 			if (HandleSolved(movesDsp, tiles, faces, orient, middle))
-				(void) sprintf(messageDsp, "Congratulations %s!!", getlogin());
+				(void) sprintf(messageDsp, "Congratulations %s!!", usernameDsp);
 			else
 				(void) strcpy(messageDsp, "Solved!");
 			XtSetArg(arg[0], XtNstart, FALSE);
@@ -482,10 +516,12 @@ PrintRecord(int tiles, int faces, Boolean orient, Boolean middle)
 
 	if (tiles > MAXTILES)
 		motif_print(record, "NOT RECORDED");
-	else if (mlinkRecord[l][k][j][i] >= MAXRECORD)
-		motif_print(record, "NEVER");
-	else {
-		(void) sprintf(buff, "%d", mlinkRecord[l][k][j][i]);
+	else if (mlinkRecord[l][k][j][i].score >= MAXRECORD) {
+		(void) sprintf(buff, "NEVER %s", NOACCESS);
+		motif_print(record, buff);
+	} else {
+		(void) sprintf(buff, "%d %s",
+		mlinkRecord[l][k][j][i].score, mlinkRecord[l][k][j][i].name);
 		motif_print(record, buff);
 	}
 }
@@ -496,10 +532,13 @@ HandleSolved(int counter, int tiles, int faces, Boolean orient, Boolean middle)
 	int         i = tiles - MINTILES, j = faces - MINFACES;
 	int         k = (orient) ? 1 : 0, l = (middle) ? 1 : 0;
 
-	if (tiles <= MAXTILES && counter < mlinkRecord[l][k][j][i]) {
-		mlinkRecord[l][k][j][i] = counter;
-		if (tiles < 4 || faces < 2)
-			mlinkRecord[!l][k][j][i] = counter;
+	if (tiles <= MAXTILES && counter < mlinkRecord[l][k][j][i].score) {
+		mlinkRecord[l][k][j][i].score = counter;
+		(void) strcpy(mlinkRecord[l][k][j][i].name, usernameDsp);
+		if (tiles < 4 || faces < 2) {
+			mlinkRecord[!l][k][j][i].score = counter;
+			(void) strcpy(mlinkRecord[!l][k][j][i].name, usernameDsp);
+		}
 		WriteRecords();
 		PrintRecord(tiles, faces, orient, middle);
 		return TRUE;
@@ -508,16 +547,26 @@ HandleSolved(int counter, int tiles, int faces, Boolean orient, Boolean middle)
 }
 
 static void
-ReadRecords(void)
+InitRecords(void)
 {
-	FILE       *fp;
-	int         i, j, k, l, n;
+	int         i, j, k, l;
 
 	for (l = 0; l < 2; l++)
 		for (k = 0; k < 2; k++)
 			for (j = 0; j < MAXFACES - MINFACES + 1; j++)
-				for (i = 0; i < MAXTILES - MINTILES + 1; i++)
-					mlinkRecord[l][k][j][i] = MAXRECORD;
+				for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
+					mlinkRecord[l][k][j][i].score = MAXRECORD;
+					(void) strcpy(mlinkRecord[l][k][j][i].name, NOACCESS);
+				}
+}
+
+static void
+ReadRecords(void)
+{
+	FILE       *fp;
+	int         i, j, k, l, n;
+	char        username[USERNAMELEN];
+
 	if ((fp = fopen(SCOREFILE, "r")) == NULL) {
 		(void) sprintf(buff, "Can not open %s, taking defaults.", SCOREFILE);
 		motif_print(message, buff);
@@ -526,8 +575,11 @@ ReadRecords(void)
 			for (k = 0; k < 2; k++)
 				for (j = 0; j < MAXFACES - MINFACES + 1; j++)
 					for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
-						(void) fscanf(fp, "%d", &n);
-						mlinkRecord[l][k][j][i] = n;
+						(void) fscanf(fp, "%d %s\n", &n, username);
+						if (n <= mlinkRecord[l][k][j][i].score) {
+							mlinkRecord[l][k][j][i].score = n;
+							(void) strcpy(mlinkRecord[l][k][j][i].name, username);
+						}
 					}
 		(void) fclose(fp);
 	}
@@ -539,21 +591,46 @@ WriteRecords(void)
 	FILE       *fp;
 	int         i, j, k, l;
 
+	ReadRecords();		/* Maybe its been updated by another */
 	if ((fp = fopen(SCOREFILE, "w")) == NULL) {
 		(void) sprintf(buff, "Can not write to %s.", SCOREFILE);
 		motif_print(message, buff);
 	} else {
+#if HAVE_FCNTL_H
+		int         lfd;
+		char        lockfile[FILENAMELEN];
+
+		(void) strcpy(lockfile, SCOREFILE);
+		(void) strcat(lockfile, ".lock");
+		while (((lfd = open(lockfile, O_CREAT | O_EXCL, 0644)) < 0) &&
+		       errno == EEXIST)
+			(void) sleep(1);
+		if (lfd < 0) {
+#if 1
+			(void) fprintf(stderr, "Lock file exists... guessing its an old one.\n");
+#else
+			(void) fprintf(stderr, "Lock file exists... score not recorded - sorry.\n"
+				);
+			return;
+#endif
+		}
+#endif
 		for (l = 0; l < 2; l++) {
 			for (k = 0; k < 2; k++) {
 				for (j = 0; j < MAXFACES - MINFACES + 1; j++) {
 					for (i = 0; i < MAXTILES - MINTILES + 1; i++)
-						(void) fprintf(fp, "%d ", mlinkRecord[l][k][j][i]);
+						(void) fprintf(fp, "%d %s\n",
+							       mlinkRecord[l][k][j][i].score, mlinkRecord[l][k][j][i].name);
 					(void) fprintf(fp, "\n");
 				}
 				(void) fprintf(fp, "\n");
 			}
 			(void) fprintf(fp, "\n");
 		}
+#if HAVE_FCNTL_H
+		(void) close(lfd);
+		(void) unlink(lockfile);
+#endif
 		(void) fclose(fp);
 	}
 }
