@@ -6,7 +6,7 @@
 #
 ###
 #
-#  Copyright (c) 1993 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1993 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -34,14 +34,20 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 #ifdef VMS
 #include <unixlib.h>
-#define getlogin cuserid
+#define getlogin() cuserid(NULL)
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
+#if HAVE_FCNTL_H
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#endif
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
@@ -71,12 +77,25 @@
 
 #define MAXOCTAS 6
 #define MAXRECORD 32767
+#define FILENAMELEN 1024
+#define USERNAMELEN 128
+#define NOACCESS "noaccess"
+#define NOBODY "nobody"
+
+typedef struct {
+	int         score;
+	char        name[USERNAMELEN];
+} GameRecord;
 
 static void Initialize(Widget w);
-static void CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData);
+static void CallbackOct(Widget w, caddr_t clientData,
+			octCallbackStruct * callData);
 
-static void PrintRecord(int size, int mode, Boolean orient, Boolean sticky, Boolean practice);
-static int  HandleSolved(int counter, int size, int mode, Boolean orient, Boolean sticky);
+static void PrintRecord(int size, int mode,
+			Boolean orient, Boolean sticky, Boolean practice);
+static int  HandleSolved(int counter, int size, int mode,
+			 Boolean orient, Boolean sticky);
+static void InitRecords(void);
 static void ReadRecords(void);
 static void WriteRecords(void);
 
@@ -90,9 +109,11 @@ static void motif_print(Widget w, char *text);
 static Arg  arg[5];
 static Widget moves, record, message, oct, modes[MAXMODES], orientSwitch,
             stickySwitch, practiceSwitch, octas;
-static int  octRecord[MAXMODES][2][MAXOCTAS - MINOCTAS + 2], movesDsp = 0;
+static GameRecord octRecord[MAXMODES][2][MAXOCTAS - MINOCTAS + 2];
+static int  movesDsp = 0;
 static int  oldSize;
 static char messageDsp[128] = "Welcome";
+static char usernameDsp[USERNAMELEN] = "";
 static char buff[256];
 
 static char *modeString[] =
@@ -107,25 +128,35 @@ Usage(void)
 	(void) fprintf(stderr,
 	     "\t[-geometry [{width}][x{height}][{+-}{xoff}[{+-}{yoff}]]]\n");
 	(void) fprintf(stderr,
-	       "\t[-display [{host}]:[{vs}]] [-fg {color}] [-bg {color}]\n");
+		       "\t[-display [{host}]:[{vs}]] [-mono]\n");
 	(void) fprintf(stderr,
-	      "\t[-{border|bd} {color}] [-mono] [-{size {int} | sticky}]\n");
+		"\t[-{foreground|fg} {color}] [-{background|bg} {color}]\n");
 	(void) fprintf(stderr,
-		"\t[[-{mode {int} | both}] [-[no]orient] [-[no]practice]\n");
+	      "\t[-{border|bd} {color}] [-face{0|1|2|3|4|5|6|7} {color}]\n");
 	(void) fprintf(stderr,
-		       "\t[-face{0|1|2|3|4|5|6|7} {color}]\n");
+		       "\t[-{size {int} | sticky}] [-{mode {int} | both}]\n");
+	(void) fprintf(stderr,
+		   "\t[-[no]orient] [-[no]practice] [-username {string}]\n");
 	exit(1);
 }
 
 static XrmOptionDescRec options[] =
 {
-	{"-fg", "*oct.Foreground", XrmoptionSepArg, NULL},
-	{"-bg", "*Background", XrmoptionSepArg, NULL},
-	{"-foreground", "*oct.Foreground", XrmoptionSepArg, NULL},
-	{"-background", "*Background", XrmoptionSepArg, NULL},
-	{"-border", "*oct.pieceBorder", XrmoptionSepArg, NULL},
-	{"-bd", "*oct.pieceBorder", XrmoptionSepArg, NULL},
 	{"-mono", "*oct.mono", XrmoptionNoArg, "TRUE"},
+	{"-fg", "*oct.Foreground", XrmoptionSepArg, NULL},
+	{"-foreground", "*oct.Foreground", XrmoptionSepArg, NULL},
+	{"-bg", "*Background", XrmoptionSepArg, NULL},
+	{"-background", "*Background", XrmoptionSepArg, NULL},
+	{"-bd", "*oct.pieceBorder", XrmoptionSepArg, NULL},
+	{"-border", "*oct.pieceBorder", XrmoptionSepArg, NULL},
+	{"-face0", "*oct.faceColor0", XrmoptionSepArg, NULL},
+	{"-face1", "*oct.faceColor1", XrmoptionSepArg, NULL},
+	{"-face2", "*oct.faceColor2", XrmoptionSepArg, NULL},
+	{"-face3", "*oct.faceColor3", XrmoptionSepArg, NULL},
+	{"-face4", "*oct.faceColor4", XrmoptionSepArg, NULL},
+	{"-face5", "*oct.faceColor5", XrmoptionSepArg, NULL},
+	{"-face6", "*oct.faceColor6", XrmoptionSepArg, NULL},
+	{"-face7", "*oct.faceColor7", XrmoptionSepArg, NULL},
 	{"-size", "*oct.size", XrmoptionSepArg, NULL},
 	{"-sticky", "*oct.sticky", XrmoptionNoArg, "FALSE"},
 	{"-mode", "*oct.mode", XrmoptionSepArg, NULL},
@@ -134,14 +165,7 @@ static XrmOptionDescRec options[] =
 	{"-noorient", "*oct.orient", XrmoptionNoArg, "FALSE"},
 	{"-practice", "*oct.practice", XrmoptionNoArg, "TRUE"},
 	{"-nopractice", "*oct.practice", XrmoptionNoArg, "FALSE"},
-	{"-face0", "*oct.faceColor0", XrmoptionSepArg, NULL},
-	{"-face1", "*oct.faceColor1", XrmoptionSepArg, NULL},
-	{"-face2", "*oct.faceColor2", XrmoptionSepArg, NULL},
-	{"-face3", "*oct.faceColor3", XrmoptionSepArg, NULL},
-	{"-face4", "*oct.faceColor4", XrmoptionSepArg, NULL},
-	{"-face5", "*oct.faceColor5", XrmoptionSepArg, NULL},
-	{"-face6", "*oct.faceColor6", XrmoptionSepArg, NULL},
-	{"-face7", "*oct.faceColor7", XrmoptionSepArg, NULL}
+	{"-username", "*oct.userName", XrmoptionSepArg, NULL}
 };
 
 int
@@ -278,11 +302,13 @@ Initialize(Widget w)
 {
 	int         size, mode;
 	Boolean     orient, sticky, practice;
+	String      username;
 
 	XtVaSetValues(w,
 		      XtNstart, FALSE,
 		      NULL);
 	XtVaGetValues(w,
+		      XtNuserName, &username,
 		      XtNsize, &size,
 		      XtNmode, &mode,
 		      XtNorient, &orient,
@@ -295,7 +321,16 @@ Initialize(Widget w)
 	XmToggleButtonSetState(orientSwitch, orient, True);
 	XmToggleButtonSetState(stickySwitch, sticky, True);
 	XmToggleButtonSetState(practiceSwitch, practice, True);
+	InitRecords();
 	ReadRecords();
+	(void) strcpy(usernameDsp, username);
+	if (!strcmp(usernameDsp, "") || !strcmp(usernameDsp, NOACCESS) ||
+	    !strcmp(usernameDsp, NOBODY)) {
+		/* The NOACCESS is not necasary, but it stops people from being cute. */
+		(void) sprintf(usernameDsp, "%s", getlogin());
+		if (!strcmp(usernameDsp, "") || !strcmp(usernameDsp, NOACCESS))
+			(void) sprintf(usernameDsp, "%s", NOBODY);	/* It really IS nobody */
+	}
 	PrintRecord(size, mode, orient, sticky, practice);
 	oldSize = size;
 }
@@ -345,7 +380,7 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 				movesDsp = 0;
 			else {
 				if (HandleSolved(movesDsp, size, mode, orient, sticky))
-					(void) sprintf(messageDsp, "Congratulations %s!!", getlogin());
+					(void) sprintf(messageDsp, "Congratulations %s!!", usernameDsp);
 				else
 					(void) strcpy(messageDsp, "Solved!");
 			}
@@ -577,10 +612,12 @@ PrintRecord(int size, int mode, Boolean orient, Boolean sticky, Boolean practice
 		motif_print(record, "practice");
 	else if (!sticky && size > MAXOCTAS)
 		motif_print(record, "NOT RECORDED");
-	else if (octRecord[i][j][k] >= MAXRECORD)
-		motif_print(record, "NEVER");
-	else {
-		(void) sprintf(buff, "%d", octRecord[i][j][k]);
+	else if (octRecord[i][j][k].score >= MAXRECORD) {
+		(void) sprintf(buff, "NEVER %s", NOACCESS);
+		motif_print(record, buff);
+	} else {
+		(void) sprintf(buff, "%d %s",
+			  octRecord[i][j][k].score, octRecord[i][j][k].name);
 		motif_print(record, buff);
 	}
 }
@@ -592,11 +629,14 @@ HandleSolved(int counter, int size, int mode, Boolean orient, Boolean sticky)
 	int         j = (orient) ? 1 : 0;
 	int         k = (sticky) ? MAXOCTAS - MINOCTAS + 1 : size - MINOCTAS;
 
-	if ((sticky || size <= MAXOCTAS) && counter < octRecord[i][j][k]) {
-		octRecord[i][j][k] = counter;
+	if ((sticky || size <= MAXOCTAS) && counter < octRecord[i][j][k].score) {
+		octRecord[i][j][k].score = counter;
+		(void) strcpy(octRecord[i][j][k].name, usernameDsp);
 		if ((size < 2 && mode != PERIOD4) || (size < 4 && mode == PERIOD4) ||
-		    (orient && (octRecord[i][j][k] < octRecord[i][!j][k])))
-			octRecord[i][!j][k] = counter;
+		    (orient && (counter < octRecord[i][!j][k].score))) {
+			octRecord[i][!j][k].score = counter;
+			(void) strcpy(octRecord[i][!j][k].name, usernameDsp);
+		}
 		WriteRecords();
 		PrintRecord(size, mode, orient, sticky, FALSE);
 		return TRUE;
@@ -604,16 +644,26 @@ HandleSolved(int counter, int size, int mode, Boolean orient, Boolean sticky)
 	return FALSE;
 }
 
+static void
+InitRecords(void)
+{
+	int         i, mode, orient;
+
+	for (mode = 0; mode < MAXMODES; mode++)
+		for (orient = 0; orient < 2; orient++)
+			for (i = 0; i <= MAXOCTAS - MINOCTAS + 1; i++) {
+				octRecord[mode][orient][i].score = MAXRECORD;
+				(void) strcpy(octRecord[mode][orient][i].name, NOACCESS);
+			}
+}
+
 static void
 ReadRecords(void)
 {
 	FILE       *fp;
 	int         i, n, mode, orient;
+	char        username[USERNAMELEN];
 
-	for (mode = 0; mode < MAXMODES; mode++)
-		for (orient = 0; orient < 2; orient++)
-			for (i = 0; i <= MAXOCTAS - MINOCTAS + 1; i++)
-				octRecord[mode][orient][i] = MAXRECORD;
 	if ((fp = fopen(SCOREFILE, "r")) == NULL) {
 		(void) sprintf(buff, "Can not open %s, taking defaults.", SCOREFILE);
 		motif_print(message, buff);
@@ -621,8 +671,11 @@ ReadRecords(void)
 		for (mode = 0; mode < MAXMODES; mode++)
 			for (orient = 0; orient < 2; orient++)
 				for (i = 0; i <= MAXOCTAS - MINOCTAS + 1; i++) {
-					(void) fscanf(fp, "%d", &n);
-					octRecord[mode][orient][i] = n;
+					(void) fscanf(fp, "%d %s\n", &n, username);
+					if (n <= octRecord[mode][orient][i].score) {
+						octRecord[mode][orient][i].score = n;
+						(void) strcpy(octRecord[mode][orient][i].name, username);
+					}
 				}
 		(void) fclose(fp);
 	}
@@ -634,18 +687,43 @@ WriteRecords(void)
 	FILE       *fp;
 	int         i, mode, orient;
 
+	ReadRecords();		/* Maybe its been updated by another */
 	if ((fp = fopen(SCOREFILE, "w")) == NULL) {
 		(void) sprintf(buff, "Can not write to %s.", SCOREFILE);
 		motif_print(message, buff);
 	} else {
+#if HAVE_FCNTL_H
+		int         lfd;
+		char        lockfile[FILENAMELEN];
+
+		(void) strcpy(lockfile, SCOREFILE);
+		(void) strcat(lockfile, ".lock");
+		while (((lfd = open(lockfile, O_CREAT | O_EXCL, 0644)) < 0) &&
+		       errno == EEXIST)
+			(void) sleep(1);
+		if (lfd < 0) {
+#if 1
+			(void) fprintf(stderr, "Lock file exists... guessing its an old one.\n");
+#else
+			(void) fprintf(stderr, "Lock file exists... score not recorded - sorry.\n"
+				);
+			return;
+#endif
+		}
+#endif
 		for (mode = 0; mode < MAXMODES; mode++) {
 			for (orient = 0; orient < 2; orient++) {
 				for (i = 0; i <= MAXOCTAS - MINOCTAS + 1; i++)
-					(void) fprintf(fp, "%d ", octRecord[mode][orient][i]);
+					(void) fprintf(fp, "%d %s\n",
+						       octRecord[mode][orient][i].score, octRecord[mode][orient][i].name);
 				(void) fprintf(fp, "\n");
 			}
 			(void) fprintf(fp, "\n");
 		}
+#if HAVE_FCNTL_H
+		(void) close(lfd);
+		(void) unlink(lockfile);
+#endif
 		(void) fclose(fp);
 	}
 }
