@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -30,7 +30,7 @@
 #ifdef VMS
 #include <unixlib.h>
 #else
-#ifndef apollo
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #endif
@@ -200,6 +200,8 @@ static XtActionsRec actionsListPyraminx[] =
 
 static XtResource resourcesPyraminx[] =
 {
+	{XtNuserName, XtCUserName, XtRString, sizeof (String),
+	 XtOffset(PyraminxWidget, pyraminx.username), XtRString, "nobody"},
   /* Beware color values are swapped */
 	{XtNfaceColor0, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(PyraminxWidget, pyraminx.faceName[0]), XtRString, "Blue"},
