@@ -454,11 +454,11 @@ DrawSquare2D(Rubik2DWidget w, int face, int position, int offset)
 		XFillRectangle(XtDisplay(w), XtWindow(w),
 			       w->rubik.borderGC, dx, dy, w->rubik2d.cubeLength, w->rubik2d.cubeLength);
 		XDrawRectangle(XtDisplay(w), XtWindow(w),
-		      w->rubik.faceGC[w->rubik.cubeLoc[face][position].face],
+		      w->rubik.faceGC[(int) w->rubik.cubeLoc[face][position].face],
 		       dx, dy, w->rubik2d.cubeLength, w->rubik2d.cubeLength);
 	} else {
 		XFillRectangle(XtDisplay(w), XtWindow(w),
-		      w->rubik.faceGC[w->rubik.cubeLoc[face][position].face],
+		      w->rubik.faceGC[(int) w->rubik.cubeLoc[face][position].face],
 		       dx, dy, w->rubik2d.cubeLength, w->rubik2d.cubeLength);
 		XDrawRectangle(XtDisplay(w), XtWindow(w),
 			       w->rubik.borderGC, dx, dy, w->rubik2d.cubeLength, w->rubik2d.cubeLength);
@@ -468,7 +468,7 @@ DrawSquare2D(Rubik2DWidget w, int face, int position, int offset)
 		char        buf[2];
 
 		(void) sprintf(buf, "%c",
-		w->rubik.faceName[w->rubik.cubeLoc[face][position].face][0]);
+		w->rubik.faceName[(int) w->rubik.cubeLoc[face][position].face][0]);
 		letterX = dx + w->rubik2d.cubeLength / 2 + w->rubik.letterOffset.x;
 		letterY = dy + w->rubik2d.cubeLength / 2 + w->rubik.letterOffset.y;
 		XDrawString(XtDisplay(w), XtWindow(w), w->rubik.inverseGC,
