@@ -6,7 +6,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 96	David Albert Bagley, bagleyd@hertz.njit.edu
+#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
 #
 #                   All Rights Reserved
 #
@@ -126,6 +126,8 @@ static XtActionsRec actionsListSkewb3D[] =
 
 static XtResource resourcesSkewb3D[] =
 {
+	{XtNuserName, XtCUserName, XtRString, sizeof (String),
+	 XtOffset(SkewbWidget, skewb.username), XtRString, "nobody"},
 	{XtNfaceColor0, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.faceName[0]), XtRString, "Red"},
 	{XtNfaceColor1, XtCLabel, XtRString, sizeof (String),
